# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, fields, models
from odoo.exceptions import UserError


class CoSavoirsSaleOrder(models.Model):
    _name = "sale.order"
    _inherit = ["sale.order", "digiforma.api"]

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    digiforma_session_ids = fields.One2many(
        comodel_name="digiforma.session.data",
        inverse_name="sale_order_id",
        string="Sessions associées",
        required=False,
    )

    count_digiforma_session = fields.Integer(
        "Sessions digiforma associées", compute="_compute_count_digiforma_session"
    )

    # ------------------------------------------------------
    # Compute fields
    # ------------------------------------------------------
    def _compute_count_digiforma_session(self):
        for so in self:
            so.count_digiforma_session = len(so.digiforma_session_ids)

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_confirm(self):
        """
        When a sale order is confirmed :
            - we check if company exists in Digiforma
            - we check articles conf
            (is_quantity_independent, is_processed_externally)
            - we create one or several sessions in Digiforma
        :return: super action_confirm
        """
        for so in self:
            # check if product digiforma in SOL
            digi_sol = so.order_line.filtered("product_id.is_formation_digi")
            if digi_sol:
                digiforma_partner = so.digiforma_fetch_company()
                if not digiforma_partner:
                    raise UserError(_("Ce client n'est pas configuré avec Digiforma"))
                else:
                    so.create_digiforma_sessions_so()
            return super(CoSavoirsSaleOrder, so).action_confirm()

    def show_digiforma_sessions_so(self):
        """
        Return sessions linked to sale order
        :return: list of dict
        """
        sessions_to_show = list()
        all_sessions_digiforma = self.digiforma_get_all_sessions()

        for sol in self.order_line:
            if sol.digiforma_session_ids:
                sessions_to_show.append(
                    sol.digiforma_session_ids.get_sessions_to_display(
                        *all_sessions_digiforma
                    )
                )
        return sessions_to_show

    def delete_digiforma_sessions_so(self):
        """
        Delete digiforma sessions linked to so
        :return:
        """
        sessions_digiforma_linked = self.digiforma_session_ids
        for session in sessions_digiforma_linked:
            if self.digiforma_delete_sessions(session.digiforma_id):
                session.unlink()
            else:
                raise UserError(_("La suppression des sessions n'a pas fonctionné"))

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def create_digiforma_sessions_so(self):
        """
        Create sessions in digiforma and sessions stored in Odoo
        :return:
        """
        so_line_digiforma = self.mapped("order_line").filtered(
            lambda sol: sol.product_id.is_formation_digi
        )
        for so_line in so_line_digiforma:
            if len(so_line.digiforma_session_ids) == 0:
                product = so_line.product_id
                vat = float(0)
                if product.is_quantity_independent:
                    qty = 1
                    cost = so_line.price_subtotal
                else:
                    qty = int(so_line.product_uom_qty)
                    cost = so_line.price_unit
                session_variables = {
                    "name": {"value": "Odoo - " + product.name},
                    "pipelineState": {"value": "incomplete"},
                    "contracted": {
                        "value": True if product.is_contracted else False,
                        "type": "bool",
                    },
                    "costs": {
                        "value": [
                            {
                                "cost": {"value": cost},
                                "costIndependant": {"value": cost},
                                "costIndividual": {"value": cost},
                                "description": {"value": "Formation"},
                                "vat": {"value": vat},
                            },
                        ],
                        "type": "object",
                    },
                }
                for _i in range(0, qty, 1):
                    digiforma_session_id = self.digiforma_create_session(
                        session_variables
                    )
                    customer_variables = {
                        "companyId": {"value": int(self.partner_id.digiforma_id)},
                        "crmStatus": {"value": "WON", "type": "enum"},
                        "trainingSessionId": {"value": digiforma_session_id},
                    }
                    self.digiforma_create_customer(customer_variables)
                    self.env["digiforma.session.data"].create(
                        {
                            "digiforma_id": digiforma_session_id,
                            "name": product.name,
                            "sale_order_id": self.id,
                            "sale_order_line_id": so_line.id,
                        }
                    ).env.cr.commit()

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    def unlink(self):
        for so in self:
            if so.digiforma_session_ids:
                raise UserError(
                    _(
                        "Vous ne pouvez pas supprimer un devis rattaché à une "
                        "ou des sessions Digiforma : \n Vous pouvez l'annuler "
                        "ou supprimer les sessions associées."
                    )
                )
        return super().unlink()
